package com.ningdatech.pmapi.common.helper.basic;

import com.github.benmanes.caffeine.cache.Caffeine;
import com.github.benmanes.caffeine.cache.LoadingCache;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.pmapi.common.helper.RegionCacheHelper;
import com.ningdatech.pmapi.common.model.RegionMapKey;
import com.ningdatech.pmapi.sys.convert.RegionConverter;
import com.ningdatech.pmapi.sys.model.dto.RegionDTO;
import com.ningdatech.pmapi.sys.model.dto.RegionTreeDTO;
import com.ningdatech.pmapi.sys.service.IRegionService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * <p>
 * AbstractRegionCache
 * </p>
 *
 * @author WendyYang
 * @since 14:41 2023/3/1
 */
public abstract class AbstractRegionCache implements InitializingBean, RegionCacheHelper {

    @Autowired
    private IRegionService regionService;
    /**
     * 当前支持最大层级
     */
    private static final int REGION_LEVEL_MAX = 3;

    private LoadingCache<String, List<RegionTreeDTO>> regionsCache;

    /**
     * 中国行政区划编码
     */
    private static final String PARENT_CODE_ROOT = "100000";

    protected Map<RegionMapKey, RegionDTO> regionMap;

    @Override
    public void afterPropertiesSet() {
        regionsCache = Caffeine.newBuilder()
                .expireAfterWrite(10, TimeUnit.MINUTES)
                .refreshAfterWrite(5, TimeUnit.MINUTES)
                .build(key -> {
                    // 查询全部
                    List<RegionDTO> regionDTOList = regionService.all();
                    if (regionDTOList.isEmpty()) {
                        throw BizException.wrap("区域元数据不能为空");
                    }
                    regionMap = buildCacheMap(regionDTOList);
                    List<RegionDTO> regionInLevel = regionMap.values().stream()
                            .filter(regionDTO -> {
                                // 只过滤出小于等于level的region
                                if (Objects.isNull(regionDTO.getRegionLevel())) {
                                    return false;
                                }
                                return Integer.parseInt(key) >= regionDTO.getRegionLevel();
                            }).collect(Collectors.toList());
                    List<RegionTreeDTO> treeDtos = RegionConverter.toRegionTreeDTOList(regionInLevel);
                    return toTreeStructure(treeDtos);
                });
        warmUp();
    }

    protected List<RegionTreeDTO> getByLevel(int level) {
        return regionsCache.get(String.valueOf(level));
    }

    private Map<RegionMapKey, RegionDTO> buildCacheMap(List<RegionDTO> regionDTOList) {
        Map<RegionMapKey, RegionDTO> regionDtoMap = new ConcurrentHashMap<>(256);
        regionDTOList.forEach(region -> {
            RegionMapKey key = RegionMapKey.builder()
                    .regionCode(region.getRegionCode())
                    .regionLevel(region.getRegionLevel())
                    .build();
            regionDtoMap.put(key, region);
        });
        return regionDtoMap;
    }

    private List<RegionTreeDTO> toTreeStructure(List<RegionTreeDTO> rootList) {
        List<RegionTreeDTO> nodeList = new ArrayList<>();
        for (RegionTreeDTO treeNode : rootList) {
            if (PARENT_CODE_ROOT.equals(treeNode.getParentCode()) || Objects.isNull(treeNode.getParentCode())) {
                nodeList.add(treeNode);
            }
            treeNode.setChildren(getChildren(treeNode.getRegionCode(), treeNode.getLevel(), rootList));
        }
        return nodeList;
    }

    private List<RegionTreeDTO> getChildren(String regionCode, int regionLevel, List<RegionTreeDTO> list) {
        List<RegionTreeDTO> childList = new ArrayList<>();
        for (RegionTreeDTO regionTreeDTO : list) {
            if (regionCode.equals(regionTreeDTO.getParentCode()) && regionLevel + 1 == regionTreeDTO.getLevel()) {
                childList.add(regionTreeDTO);
            }
        }
        for (RegionTreeDTO regionTreeDTO : childList) {
            regionTreeDTO.setChildren(getChildren(regionTreeDTO.getRegionCode(), regionTreeDTO.getLevel(), list));
        }
        if (CollectionUtils.isEmpty(childList)) {
            return Collections.emptyList();
        }
        childList.sort(Comparator.comparing(RegionTreeDTO::getRegionCode));
        return childList;
    }


    private void warmUp() {
        for (int level = 1; level <= REGION_LEVEL_MAX; level++) {
            getByLevel(level);
        }
    }

    protected List<RegionTreeDTO> getCopyListByLevel(int level) {
        List<RegionTreeDTO> regionTreeDtos = regionsCache.get(String.valueOf(level));
        List<RegionTreeDTO> copyRegionTreeDtos = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(regionTreeDtos)) {
            copyRegionTreeDtos.addAll(regionTreeDtos);
        }
        return copyRegionTreeDtos;
    }


    @Override
    public String getParentCodeRoot() {
        return PARENT_CODE_ROOT;
    }
}
