package com.ningdatech.pmapi.common.helper.impl;

import cn.hutool.core.lang.Assert;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.common.helper.basic.AbstractRegionCache;
import com.ningdatech.pmapi.common.model.RegionMapKey;
import com.ningdatech.pmapi.sys.model.dto.RegionDTO;
import com.ningdatech.pmapi.sys.model.dto.RegionTreeDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author liuxinxin
 * @date 2022/7/22 上午8:58
 * 构建地区码 地区树
 */
@Slf4j
@Component
public class RegionsCacheImpl extends AbstractRegionCache {

    @Override
    public List<RegionTreeDTO> all() {
        return getByLevel(3);
    }

    @Override
    public List<String> getAllChildrenRegionCodeList(String code, int level) {
        List<RegionTreeDTO> regionTreeDTOList = all();
        Set<String> childrenRegionCodeSet = new HashSet<>();
        childrenRegionCodeSet.add(code);
        RegionTreeDTO currentRegionTree = getCurrentRegionTree(code, level, regionTreeDTOList);
        if (Objects.isNull(currentRegionTree)) {
            List<String> childrenRegionCodeList = new ArrayList<>();
            childrenRegionCodeList.add(code);
            return childrenRegionCodeList;
        }
        getAllChildrenRegionCodeList(currentRegionTree.getChildren(), childrenRegionCodeSet);
        return new ArrayList<>(childrenRegionCodeSet);
    }

    private RegionTreeDTO getCurrentRegionTree(String code, int level, List<RegionTreeDTO> regionTreeDtos) {
        for (RegionTreeDTO regionTreeDTO : regionTreeDtos) {
            if (level == regionTreeDTO.getLevel() && code.equals(regionTreeDTO.getRegionCode())) {
                return regionTreeDTO;
            }
            if (CollectionUtils.isNotEmpty(regionTreeDTO.getChildren())) {
                return getCurrentRegionTree(code, level, regionTreeDTO.getChildren());
            }
        }
        return null;
    }

    private void getAllChildrenRegionCodeList(List<RegionTreeDTO> regionTreeDtos, Set<String> childrenRegionCodeSet) {
        if (CollectionUtils.isEmpty(regionTreeDtos)) {
            return;
        }
        for (RegionTreeDTO regionTreeDTO : regionTreeDtos) {
            childrenRegionCodeSet.add(regionTreeDTO.getRegionCode());
            getAllChildrenRegionCodeList(regionTreeDTO.getChildren(), childrenRegionCodeSet);

        }
    }

    @Override
    public List<String> getRegionCodes(String regionCode, int regionLevel) {
        RegionTreeDTO regionTreeNode = getTreeByRegionAndCode(null, regionCode, regionLevel);
        Assert.notNull(regionTreeNode, "不存在此级别区域信息：{}", regionLevel);
        List<String> regionCodes = new ArrayList<>();
        if (regionTreeNode != null) {
            regionCodes.addAll(CollUtils.fieldList(treeToList(Collections.singletonList(regionTreeNode)), RegionTreeDTO::getRegionCode));
        }
        if (!regionCodes.contains(regionCode)) {
            regionCodes.add(regionCode);
        }
        return regionCodes;
    }

    protected List<RegionTreeDTO> treeToList(List<RegionTreeDTO> treeList) {
        ArrayList<RegionTreeDTO> result = new ArrayList<>();
        treeList.forEach(w -> {
            result.add(w);
            if (CollectionUtils.isNotEmpty(w.getChildren())) {
                result.addAll(treeToList(w.getChildren()));
            }
        });
        return result;
    }


    /**
     * 获取某一个地区开始的层级树
     *
     * @param list  地区集合
     * @param code  地区编码
     * @param level 地区层级
     * @return /
     */
    protected RegionTreeDTO getTreeByRegionAndCode(List<RegionTreeDTO> list, String code, int level) {
        if (CollectionUtils.isEmpty(list)) {
            list = super.getCopyListByLevel(3);
            if (CollectionUtils.isEmpty(list)) {
                return null;
            }
        }
        Optional<RegionTreeDTO> first = list.stream()
                .filter(w -> w.getRegionCode().equals(code) && w.getLevel() == level)
                .findFirst();
        if (first.isPresent()) {
            return first.get();
        }
        for (RegionTreeDTO dto : list) {
            if (CollectionUtils.isEmpty(dto.getChildren())) {
                continue;
            }
            RegionTreeDTO temp = getTreeByRegionAndCode(dto.getChildren(), code, level);
            if (temp != null) {
                return temp;
            }
        }
        return null;
    }

    @Override
    public List<RegionDTO> listParents(String code, int level) {
        List<RegionDTO> result = new ArrayList<>();
        RegionDTO dto = regionMap.get(RegionMapKey.of(code, level));
        result.add(0, dto);
        if (dto.getParentCode().equals(super.getParentCodeRoot())) {
            return result;
        }
        result.addAll(0, listParents(dto.getParentCode(), dto.getRegionLevel() - 1));
        return result;
    }

    @Override
    public RegionDTO getByCodeAndLevel(String code, int level) {
        return regionMap.get(RegionMapKey.of(code, level));
    }

    @Override
    public List<RegionDTO> getProvincialRegion() {
        List<RegionDTO> provincialRegionList = new ArrayList<>();
        regionMap.values().forEach(v -> {
            if (v.getRegionCode().equals(v.getParentCode()) && v.getRegionLevel() == 2) {
                provincialRegionList.add(v);
            }
        });
        return provincialRegionList;
    }

    @Override
    public List<RegionDTO> getMunicipalRegion() {
        List<RegionDTO> municipalRegionList = new ArrayList<>();
        regionMap.values().forEach(v -> {
            if (v.getRegionCode().equals(v.getParentCode()) && v.getRegionLevel() == 3) {
                municipalRegionList.add(v);
            }
        });
        return municipalRegionList;
    }

    @Override
    public Map<String, RegionDTO> getRegionMap() {
        Map<String, RegionDTO> regionDtoMap = new ConcurrentHashMap<>(512);
        regionMap.forEach((k, v) -> regionDtoMap.put(k.getRegionCode() + "###" + k.getRegionLevel(), v));
        return regionDtoMap;
    }

    @Override
    public Map<String, RegionDTO> getNameRegionMap() {
        Map<String, RegionDTO> nameRegionDtoMap = new ConcurrentHashMap<>(512);
        regionMap.forEach((k, v) -> nameRegionDtoMap.put(v.getRegionName(), v));
        return nameRegionDtoMap;
    }

    @Override
    public String getUnionPathStr(String code, Integer level) {
        if (StringUtils.isBlank(code) || Objects.isNull(level)) {
            return null;
        }
        List<String> unionPathStrList = new ArrayList<>();
        buildUnionPathStrList(code, level, unionPathStrList);
        Collections.reverse(unionPathStrList);
        if (CollectionUtils.isEmpty(unionPathStrList)) {
            return null;
        }
        return String.join("@@", unionPathStrList);
    }

    protected void buildUnionPathStrList(String code, Integer level, List<String> unionPathStrList) {
        if (level <= 0 || super.getParentCodeRoot().equals(code)) {
            return;
        }
        RegionDTO regionDTO = regionMap.get(RegionMapKey.of(code, level));
        unionPathStrList.add(regionDTO.getRegionCode() + "##" + regionDTO.getRegionName() + "##" + regionDTO.getRegionLevel());
        if (!super.getParentCodeRoot().equals(regionDTO.getParentCode())) {
            buildUnionPathStrList(regionDTO.getParentCode(), level - 1, unionPathStrList);
        }
    }
}
